@@ -2,12 +2,27 @@ from dataclasses import asdict, dataclass
 from typing import Union
 
 
+@dataclass
+class FirebaseInstallationResponse:
+    name: str
+    fid: str
+    refresh_token: str
+    auth_token: str
+    auth_token_expires_in: int
+
+
+@dataclass
+class CheckInResponse:
+    stats_ok: bool
+
+
 @dataclass
 class AppCredentialsGcm:
     token: str
-    appId: str
-    androidId: str
-    securityToken: str
+    app_id: str
+    android_id: str
+    security_token: str
+    installation: FirebaseInstallationResponse
 
 
 @dataclass
